VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "BillboardSystem"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' TrueVision 0.9
'------------------
'
' Billboard Particles system (for rain, snow, fountain,waterfall, cloud, smoke, trails...)
' ~ by Sylvain Dupont ~
'
' 02-07-2000 : Bouncing & public particle desc
' 30-06-2000 : Vertex Buffer added


Private BS As BSType
Public Event NewParticle(VelocX, VelocY, VelocZ, ColorR, ColorG, ColorB, ColorA, LifeTime, TextureIndex)




Private Type BillboardParticle
 texture As Integer
 Position As D3DVECTOR
 Direction As D3DVECTOR
 Color As D3DCOLORVALUE
 LifeTime As Single
 Key As Single
 Active As Boolean
End Type

Private Type BSType
  Tri() As D3DVERTEX
  Particle() As BillboardParticle
  Gravity As Single
  BounceFactor As Single
  ParticleCount As Long
  ParticleNow As Long
  GeneratorSpeed As Single
  Speed As Single
  Position As D3DVECTOR
  radius As Single
  EnergyDropOff As Single
  SphereCenter As D3DVECTOR
  SphereRadius As Single
  TargetPosition As D3DVECTOR
End Type

Private ComputeSphere As Boolean
Private SrcBlend, DestBlend, ChangeMode As EnumChangeMode
Private CustomPlane As D3DVECTOR, CustomPlaneDist As Single

Public Enum EnumChangeMode
  TV_CHANGE_ALPHA = 1
  TV_CHANGE_COLOR = 2
  TV_CHANGE_NO = 3
End Enum

Public Enum EnumUpdateMode
 TV_UPDATE_ACTIVE = 0
 TV_UPDATE_NON_ACTIVE = 1
 TV_UPDATE_BOTH = 2
 TV_UPDATE_GENERATOR = 4
 TV_UPDATE_RANDOM_GENERATOR = 8
 TV_UPDATE_FOLLOW_LANDSCAPE = 16
 TV_UPDATE_BOUNCE_ON_LANDSCAPE = 32
 TV_UPDATE_BOUNCE_ON_Y0_PLANE = 64
 TV_UPDATE_BOUNCE_ON_CUSTOM_PLANE = 128
End Enum

Private VertexBuffer As Direct3DVertexBuffer7
Private VBIndices() As Integer
'
Private Sub InitVertexBuffer()
  NbQuad = 25
  Dim VBDesc As D3DVERTEXBUFFERDESC
  Dim DevDesc As D3DDEVICEDESC7
  
  VBDesc.lCaps = D3DVBCAPS_WRITEONLY
  VBDesc.lFVF = D3DFVF_LVERTEX
  VBDesc.lNumVertices = NbQuad * 4
  
  Device.GetCaps DevDesc
  
  If (DevDesc.lDevCaps And D3DDEVCAPS_HWTRANSFORMANDLIGHT) = 0 Then
   'if we haven't a T&L card, copy all in the system memory
   VBDesc.lCaps = VBDesc.lCaps Or D3DVBCAPS_SYSTEMMEMORY
  End If
  
  'create the vertex buffer
  Set VertexBuffer = D3DIM.CreateVertexBuffer(VBDesc, D3DDP_DEFAULT)
  ReDim VBIndices(NbQuad * 6)
  v = 0: n = 0
  For i = 1 To NbQuad
    VBIndices(v + 0) = n + 0
    VBIndices(v + 1) = n + 3
    VBIndices(v + 2) = n + 1
    VBIndices(v + 3) = n + 1
    VBIndices(v + 4) = n + 3
    VBIndices(v + 5) = n + 2
    v = v + 6
    n = n + 4
  Next i
End Sub

Private Sub BeginVertexBuffer()
  VertexBuffer.Lock DDLOCK_WRITEONLY
  'VertexBuffer.SetVertices
End Sub


Friend Function LoadFromFile(NumFile) As Boolean
  Dim Magic As String * 4
  Get #NumFile, , Magic
  If Magic = "BBSY" Then
   Get #NumFile, , BS
  End If
End Function

Sub MoveEmettor(NewPos As D3DVECTOR)
  BS.TargetPosition = NewPos
End Sub

Sub SetBounceCustomPlane(nx, ny, nz, dist)
  CustomPlane = Vector(nx, ny, nz)
  CustomPlaneDist = dist
End Sub

Sub SetGravity(Gravity, Optional BounceFactor As Single = 0.5)
  BS.Gravity = Gravity
  BS.BounceFactor = BounceFactor
End Sub

Friend Sub WriteToFile(NumFile)
 Put #NumFile, , "BSYS"
 Put #NumFile, , BS
End Sub

Public Sub CreateBillboardSystem(ParticleCount As Integer, ParticleStart As Integer, SquareSize As Single, Position As D3DVECTOR, radius As Single, texture As texture)
  
   With BS
    .ParticleCount = ParticleCount
    ReDim .Particle(.ParticleCount)
    .Speed = 1
    ReDim .Tri(4)
    .Tri(1) = Vertex(0, -SquareSize, -SquareSize, 1, 0, 0, 1, 1)
    .Tri(2) = Vertex(0, -SquareSize, SquareSize, 1, 0, 0, 0, 1)
    .Tri(3) = Vertex(0, SquareSize, -SquareSize, 1, 0, 0, 1, 0)
    .Tri(4) = Vertex(0, SquareSize, SquareSize, 1, 0, 0, 0, 0)
    .Position = Position
    .radius = radius
    .GeneratorSpeed = 1 / 500
    .BounceFactor = 0.5
    .ParticleNow = ParticleStart
    rad2 = radius / 2
    Randomize Timer
    .TargetPosition = .Position
    For i = 1 To .ParticleCount
      If i <= .ParticleNow Then
       .Particle(i).Active = True
      Else
       .Particle(i).Active = False
      End If
    
     .Particle(i).Key = 0
     .Particle(i).LifeTime = 1
     .Particle(i).Direction = Vector(0, Rnd * 3.14, 0)
     '.Particle(i).Position = Vector(Rnd * radius - rad2, Rnd * radius - rad2, Rnd * radius - rad2)
     .Particle(i).Position = .Position
     .Particle(i).texture = texture.TextureIndex
     .Particle(i).Color = DXColor(1, 1, 1, 1)
    Next i
     AddLog "CreateBillboard System :" + CStr(ParticleCount) + " particles"
   End With
   
  
  
End Sub
Public Sub EnableComputeSphere(OnOff As Boolean)
  ComputeSphere = OnOff
End Sub


Function GetActiveParticlesNumber() As Long
  GetActiveParticlesNumber = BS.ParticleNow
End Function

Function GetParticleDirection(Particle As Long) As D3DVECTOR
  GetParticleDirection = BS.Particle(Particle).Direction
End Function
Function GetParticlePosition(Particle As Long) As D3DVECTOR
   GetParticlePosition = BS.Particle(Particle).Position
End Function

Function GetParticleKey(Particle As Long) As Single
 GetParticleKey = BS.Particle(Particle).Key
End Function

Public Sub RenderBillboardSystem()
 Dim BillMatrix As D3DMATRIX
 Dim PartMatrix As D3DMATRIX
 Dim PartMaterial As D3DMATERIAL7
 
 DirectX.IdentityMatrix BillMatrix
 'get the billboard matrix (with camera angle)
 BillMatrix = MRotate(BillMatrix, 0, -(GetCameraAngle - 90), 0)
 'BillMatrix = MTranslate(BillMatrix, BS.Position.x, BS.Position.y, BS.Position.z)

 'don't forget to set the light on
 Device.SetRenderState D3DRENDERSTATE_LIGHTING, True
 
 'set on alpha blending
 Device.SetRenderState D3DRENDERSTATE_ALPHABLENDENABLE, True
 Device.SetRenderState D3DRENDERSTATE_SRCBLEND, SrcBlend
 Device.SetRenderState D3DRENDERSTATE_DESTBLEND, DestBlend
 
 'create a blank material with variable alpha
 PartMaterial.Ambient = DXColor(1, 1, 1, 1)
 PartMaterial.Diffuse = DXColor(1, 1, 1, 1)
 
 'disable the z test --> more speed
 Device.SetRenderState D3DRENDERSTATE_ZWRITEENABLE, False
 With BS
   For i = 1 To .ParticleCount
    If .Particle(i).Active = True Then
    'translate and set the billboard matrix
    PartMatrix = MTranslate(BillMatrix, .Particle(i).Position.x, .Particle(i).Position.y, .Particle(i).Position.z)
    Device.SetTransform D3DTRANSFORMSTATE_WORLD, PartMatrix
     
    PartMaterial.Ambient = .Particle(i).Color
    PartMaterial.Diffuse = .Particle(i).Color
 
    'check if this particle texture is the same
    If OldTexture <> .Particle(i).texture Then
      OldTexture = .Particle(i).texture
      Device.SetTexture 0, texture(.Particle(i).texture)
    End If
    
    'change alpha (in order of life time)
    If ChangeMode = TV_CHANGE_ALPHA Then
     alpha = .Particle(i).Key / .Particle(i).LifeTime
     PartMaterial.Ambient.a = 1 - alpha
     PartMaterial.Diffuse.a = 1 - alpha
    End If
    If ChangeMode = TV_CHANGE_COLOR Then
     alpha = 1 - .Particle(i).Key / .Particle(i).LifeTime
     PartMaterial.Ambient = DXColor(alpha * .Particle(i).Color.r, alpha * .Particle(i).Color.g, alpha * .Particle(i).Color.b, alpha * .Particle(i).Color.a)
     PartMaterial.Diffuse = DXColor(alpha * .Particle(i).Color.r, alpha * .Particle(i).Color.g, alpha * .Particle(i).Color.b, alpha * .Particle(i).Color.a)
    End If
    Device.SetMaterial PartMaterial
    
    'draw primitives (triangle strip => square)
     Device.DrawPrimitive D3DPT_TRIANGLESTRIP, D3DFVF_VERTEX, .Tri(1), 4, D3DDP_DEFAULT
    End If
   Next i
   
 End With
 Device.SetRenderState D3DRENDERSTATE_ZWRITEENABLE, True
 
 
End Sub
Sub SetAlphaBlendingMode(Change As EnumChangeMode, Optional src As CONST_D3DBLEND = D3DBLEND_ONE, Optional dest As CONST_D3DBLEND = D3DBLEND_ONE)
  DestBlend = dest
  SrcBlend = src
  ChangeMode = Change
End Sub


Sub SetGeneralSystemPosition(Position As D3DVECTOR)
   BS.Position = Position
End Sub
Sub SetGeneratorSpeed(Speed As Single)
 BS.GeneratorSpeed = Speed
End Sub

Sub SetParticleColor(r, g, b, a, Particle)
 BS.Particle(Particle).Color = DXColor(r, g, b, a)
End Sub

Sub SetParticleDirection(Direction As D3DVECTOR, Particle)
 BS.Particle(Particle).Direction = Direction
End Sub
Sub SetParticleLifeTime(TimeLife As Single, Particle)
 BS.Particle(Particle).LifeTime = TimeLife
End Sub

Sub SetParticlePosition(Position As D3DVECTOR, Particle)
  BS.Particle(Particle).Position = Position
End Sub
Public Sub SetParticleSpeed(Speed As Single)
  BS.Speed = Speed
End Sub

Sub SetParticleTexture(texture As texture, Optional Particle As Long)
 If IsMissing(Particle) = True Then
  For i = 1 To BS.ParticleCount
    BS.Particle(i).texture = texture.TextureIndex
  Next i
 Else
  BS.Particle(Particle).texture = texture.TextureIndex
 End If
End Sub


Sub UpdateParticles(Optional UpdateMode As EnumUpdateMode = TV_UPDATE_BOTH)
 Static AncTime
 radius = BS.radius
 rad2 = radius / 2
 SpeedFactor = SystemFrame * BS.Speed
 
 If SystemFrame = 0 Or SystemFrame > 1000 Then Exit Sub
 With BS
   For i = 1 To .ParticleCount
    If .Particle(i).Active = True Then
      .Particle(i).Key = .Particle(i).Key + SystemFrame / 1000
      If .Particle(i).Key > .Particle(i).LifeTime Then
        .Particle(i).Active = False
        .Particle(i).Key = 0
        .ParticleNow = .ParticleNow - 1
      End If
      If UpdateMode And TV_UPDATE_FOLLOW_LANDSCAPE Then
      .Particle(i).Position.x = .Particle(i).Position.x + .Particle(i).Direction.x * SpeedFactor
      .Particle(i).Position.z = .Particle(i).Position.z + .Particle(i).Direction.z * SpeedFactor
      .Particle(i).Position.y = GetHeightModule(.Particle(i).Position.x + .Position.x, .Particle(i).Position.z + .Position.z)
      Else
       'change particle position      ( + direction )
      .Particle(i).Position.x = .Particle(i).Position.x + .Particle(i).Direction.x * SpeedFactor
      .Particle(i).Position.y = .Particle(i).Position.y + .Particle(i).Direction.y * SpeedFactor
      .Particle(i).Position.z = .Particle(i).Position.z + .Particle(i).Direction.z * SpeedFactor
      .Particle(i).Direction.y = .Particle(i).Direction.y - .Gravity * SystemFrame / 1000
      
      'check bounce :
      If UpdateMode And TV_UPDATE_BOUNCE_ON_LANDSCAPE Then
         If .Particle(i).Position.y < GetHeightModule(.Particle(i).Position.x, .Particle(i).Position.z) + 5 Then
           .Particle(i).Direction.y = -.Particle(i).Direction.y
           .Particle(i).Direction = VScale(.Particle(i).Direction, .BounceFactor)
         End If
      End If
      If UpdateMode And TV_UPDATE_BOUNCE_ON_CUSTOM_PLANE Then
       If DirectX.VectorDotProduct(CustomPlane, .Particle(i).Position) - CustomPlaneDist < 0 Then
            DirectX.VectorReflect .Particle(i).Direction, .Particle(i).Direction, CustomPlane
           ' .Particle(i).Direction.y = -.Particle(i).Direction.y
           .Particle(i).Direction = VScale(.Particle(i).Direction, .BounceFactor)
       End If
      End If
      If UpdateMode And TV_UPDATE_BOUNCE_ON_Y0_PLANE Then
        If .Particle(i).Position.y < 0 Then
           .Particle(i).Direction.y = -.Particle(i).Direction.y
           .Particle(i).Direction = VScale(.Particle(i).Direction, .BounceFactor)
         End If
      End If
      End If
    End If
   Next i
   
    'Simulate time :
    Dim EmettorSpeed As D3DVECTOR
    EmettorSpeed = VSubtract(.TargetPosition, .Position)
    If SystemFrame > 1000 Then SystemFrame = 1000
    LeftTime = SystemFrame + AncTime
     Do
       If LeftTime > .GeneratorSpeed Then
        'find free particle
        f = 0
        For i = 1 To .ParticleCount
          If .Particle(i).Active = False Then f = 1: Exit For
        Next i
        If f = 0 Then
          'no more free particles
          GoTo st
        End If
        If SystemFrame > 0 Then
         'the emettor have to continue to move !
        .Position = VAdd(.Position, VScale(EmettorSpeed, (SystemFrame - LeftTime) / SystemFrame))
        End If
        .ParticleNow = .ParticleNow + 1
        .Particle(i).Active = True
        .Particle(i).Key = 0
        .Particle(i).Position = .Position
        'create a new direction (and raise the event):
        Dim Veloc As D3DVECTOR, Col As D3DCOLORVALUE, LifeTime As Single
        Veloc = Vector(0, -1, 0)
        LifeTime = 1
        Col = DXColor(1, 1, 1, 1)
        textureix = .Particle(i).texture
        RaiseEvent NewParticle(Veloc.x, Veloc.y, Veloc.z, Col.r, Col.g, Col.b, Col.a, LifeTime, textureix)
        .Particle(i).Direction = Vector(Veloc.x, Veloc.y, Veloc.z)
        .Particle(i).LifeTime = LifeTime
        .Particle(i).texture = textureix
        .Particle(i).Color = Col
st:
        LeftTime = LeftTime - .GeneratorSpeed
       Else
        .Position = .TargetPosition
        AncTime = LeftTime
        LeftTime = 0
        
       End If
     Loop Until LeftTime <= 0
   End With
 
End Sub
Private Sub Class_Initialize()
 SrcBlend = D3DBLEND_ONE
 DestBlend = D3DBLEND_ONE
End Sub


