VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Landscape"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' TrueVision engine 0.9
' Landscape engine
' ------------------------
' Landscape engine is a improved version of the terrain class
' Terrain class is still in the engine for the compability with the older version
'
'
' The Landscape class has its owns sky mesh, water mesh, rain, fog & snow special effects
'
' The Landscape functions are in another module  Landscape.bas. This class is just a wrapper.
'
' Landscape is 256*256 terrain-based.

Public Enum EnumLandscapePrecision
  TV_LANDSCAPE_LOW = 32
  TV_LANDSCAPE_NORMAL = 16
  TV_LANDSCAPE_HIGH = 8
End Enum

Public Enum EnumLandscapeSun
  TV_SUN_REALTIME_MOVING = 1
  TV_SUN_CHANGE_AMBIENT_LIGHT = 2
  TV_SUN_DEFAULT = 0
End Enum

Public Enum EnumLandscapeClouds
  TV_CLOUD_MOVE = 1
  TV_CLOUD_DEFAULT = 0
End Enum
Sub GenerateNormals()
 GenerateNormalsModule
End Sub

Public Function GetHeight(x, z)
 On Error GoTo e
 GetHeight = GetHeightModule(x, z)
 Exit Function
e:
 AddLog "Error " + CStr(Err.Description)
End Function

Function GetWhetherBillboardSystem() As BillboardSystem
  Set GetWhetherBillboardSystem = WhetherBS
End Function

Sub InitClouds(CloudTexture As texture, Optional CloudFlags As EnumLandscapeClouds = TV_CLOUD_MOVE, Optional CloudHeight = 500, Optional Layer As Integer = 1, Optional LayerCount As Integer = 1)
 Land.DayFX.LandscapeCloud = True
 ReDim Preserve Land.DayFX.Layers(LayerCount)
 With Land.DayFX.Layers(Layer)
   .texture = CloudTexture.TextureIndex
   .CVertex(1) = Vertex(-256, CloudHeight, -256, 0, -1, 0, 0, 0)
   .CVertex(2) = Vertex(Land.width * 256 + 256, CloudHeight, -256, 0, -1, 0, 1, 0)
   .CVertex(3) = Vertex(-256, CloudHeight, Land.height * 256 + 256, 0, -1, 0, 0, 1)
   .CVertex(4) = Vertex(Land.width * 256 + 256, CloudHeight, Land.height * 256 + 256, 0, -1, 0, 1, 1)
 End With
Land.DayFX.CloudFlags = CloudFlags

'init a new material :
   For i = 1 To 100
    If Material(i).Active = False Then Exit For
   Next i
   Material(i).Active = True
   Material(i).Material.Ambient = DXColor(1, 1, 1, 1)
   Material(i).Material.Diffuse = DXColor(1, 1, 1, 1)
   Land.DayFX.Layers(Layer).Material = i
End Sub

Sub InitRain(Density As Single, RainTexture As texture)
    NBParticles = Density * 10
    WhetherBS.CreateBillboardSystem CInt(NBParticles), 0, 0.1, StartPos, 0, RainTexture
    For i = 1 To NBParticles
      WhetherBS.SetParticleLifeTime 5, CLng(i)
      WhetherBS.SetParticleDirection Vector(0, -1, 0), CLng(i)
      WhetherBS.SetParticleTexture RainTexture, CLng(i)
      WhetherBS.SetParticleColor 1, 1, 1, 1, CLng(i)
    Next i
    WhetherBS.SetGravity 0, 0
    WhetherBS.SetParticleSpeed 0.003
    WhetherBS.SetGeneratorSpeed 50
    WhetherBS.SetAlphaBlendingMode TV_CHANGE_NO, D3DBLEND_ONE, D3DBLEND_ONE
    LandscapeRain = True
End Sub

Sub InitSun(SunTexture As texture, Position As D3DVECTOR, Optional SunSize As Single = 1)
  Land.DayFX.LandscapeSun = True
  Land.DayFX.SunVertex(1) = Vertex(0, -200 * SunSize, -200 * SunSize, 0, 0, -1, 0, 0)
  Land.DayFX.SunVertex(2) = Vertex(0, -200 * SunSize, 200 * SunSize, 0, 0, -1, 1, 0)
  Land.DayFX.SunVertex(3) = Vertex(0, 200 * SunSize, -200 * SunSize, 0, 0, -1, 0, 1)
  Land.DayFX.SunVertex(4) = Vertex(0, 200 * SunSize, 200 * SunSize, 0, 0, -1, 1, 1)
  'init a new material :
  For i = 1 To 100
    If Material(i).Active = False Then Exit For
  Next i
  Material(i).Active = True
  Material(i).Material.Ambient = DXColor(2, 1.5, 0, 1)
  Material(i).Material.Diffuse = DXColor(2, 1.5, 0, 1)
  Material(i).Material.Specular = DXColor(2, 1.5, 0, 1)
  Land.DayFX.SunMaterial = i
  Land.DayFX.SunPosition = Position
  Land.DayFX.SunTexture = SunTexture.TextureIndex
End Sub

Sub SetWaterAltitude(Altitude As Single)
 Land.Water.Altitude = Altitude
 For i = 1 To 4
  Land.Water.Vertex(i).y = Altitude
 Next i
End Sub


Sub SetWaterEffect(alpha As Boolean, Optional AlphaFactor As Single = 0.4, Optional Animated As Boolean = False, Optional SrcBlend As CONST_D3DBLEND = D3DBLEND_SRCALPHA, Optional DestBlend As CONST_D3DBLEND = D3DBLEND_ONE)
 Land.Water.Alphablended = alpha
 Land.Water.Animated = Animated
 Land.Water.AlphaFactor = AlphaFactor
 Land.Water.SrcBlend = SrcBlend
 Land.Water.DestBlend = DestBlend
 For i = 1 To 4
  Land.Water.Vertex(i).Color = DirectX.CreateColorRGBA(1, 1, 1, AlphaFactor)
 Next i
 
End Sub

Sub SetWaterEnable(OnOff As Boolean)
 Land.Water.Enabled = True
End Sub

Sub SetWaterTexture(texture As texture)
 Land.Water.texture = texture.TextureIndex
End Sub


Sub SetWaterTextureScale(ScaleTexture As Single)
  If Land.Water.Animated = False Then
  For i = 1 To 4
   Land.Water.Vertex(i).tu = Sgn(Land.Water.Vertex(i).tu) * ScaleTexture
   Land.Water.Vertex(i).tv = Sgn(Land.Water.Vertex(i).tv) * ScaleTexture
  Next i
 
  End If
  Land.Water.TextureScale = ScaleTexture
End Sub

Sub CreateLandscape(width As Integer, height As Integer, Optional Precision As EnumLandscapePrecision = TV_LANDSCAPE_NORMAL)
 Land.Precision = Precision
 CreateLandscapeModule width, height
End Sub



Sub LoadTerrainPart(HeightMapFile As String, PositionX As Integer, PositionY As Integer)
 LoadPartTerrainModule HeightMapFile, PositionX, PositionY, 1
End Sub

Sub RenderLandscape()
 RenderLandscapeModule
End Sub

Public Sub SetDegradationDistance(DistZ As Single)
  Land.DegradationZ = DistZ
End Sub



Sub SetLandscapeMaterial(MaterialIndex As Integer)
    Land.Material = MaterialIndex
End Sub


Sub SetTerrainEnabled(PosX As Integer, PosY As Integer, OnOff As Boolean)
     NTerrain = PosY * Land.width + PosX + 1
     Land.Terrain(NTerrain).Enabled = OnOff
End Sub

Sub SetTerrainTexture(PosX As Integer, PosY As Integer, texture As texture)
   NTerrain = PosX + PosY * Land.width + 1
   Land.Terrain(NTerrain).texture = texture.TextureIndex
End Sub


Sub SetYAxisFactor(FactorY As Single)
  Land.TerrainYFactor = FactorY
End Sub


