Attribute VB_Name = "OctreeModule"
'TrueVision Engine v.8
'----------------------

'Octree module
'The octree can be used for the whole scene, so I must have a separated module

'Octree v0.5
' - tree building
' - tree rendering with ComputeSphereVisibility



'Octree Module can be get everywhere
Public Octree As OctreeType

Public Type OctreeBox
  min As D3DVECTOR
  max As D3DVECTOR
  corner(8) As D3DVECTOR
End Type

Public Type OctreeFaceType
  texture As Integer
  Vertex() As D3DVERTEX
End Type

Public Type OctreeNode
  Parent As Integer  'index to a octree node
  OctNode(8) As Integer 'index to 8 others node
  faces() As Integer 'index to octree faces
  bbox As OctreeBox
End Type

Public Type OctreeTexture
  texture As String * 40
  width As Long
  height As Long
End Type

'General octree Type
Public Type OctreeType
  Node() As OctreeNode
  Face() As OctreeFaceType
  texture() As OctreeTexture
  FaceCount As Integer
  MaxFacePerNode As Integer
End Type

Private Type OctreePlane
  normal As D3DVECTOR   ' ax + by + cz
  dist As Single        ' + d = 0
End Type

Public OctreeTextureMode As Integer, OctreeTextureWidth As Long, OctreeTextureHeight As Long
Public OctreePlanes(6) As OctreePlane

Public OctreeFaces() As Integer, NumFaceToDraw As Long
Public TextureIndex(100) As Integer
Sub GetClippingPlanes()
 '
 'get the 6 planes to do the view frustum culling. => for speed!
 '
 
 Dim RotMat As D3DMATRIX
 Dim Camera As D3DVECTOR
  
  DirectX.CopyMatrix RotMat, ViewMatrix
  DirectX.VectorCopy Camera, StartPos
  
 alpha = FOV / 2 * 3.141592 / 180
 beta = FOV / 2 * 3.141592 / 180
 
 'Get Up,Look and Right vector from the view matrix
 Dim UpVector As D3DVECTOR
 Dim LookVector As D3DVECTOR
 Dim RightVector As D3DVECTOR
 LookVector = Vector(RotMat.rc13, RotMat.rc23, RotMat.rc33)
 UpVector = Vector(RotMat.rc12, RotMat.rc22, RotMat.rc32)
 RightVector = Vector(RotMat.rc11, RotMat.rc21, RotMat.rc31)
 
 'rotate each plane with FOV angle
 Dim normal As D3DVECTOR

 'top plane
 With OctreePlanes(1)
  normal = Vector(-UpVector.x, -UpVector.y, -UpVector.z)
  DirectX.VectorRotate .normal, normal, RightVector, alpha
  .dist = DirectX.VectorDotProduct(.normal, Camera)
 End With
 
 'bottom plane
 With OctreePlanes(2)
  normal = Vector(UpVector.x, UpVector.y, UpVector.z)
  DirectX.VectorRotate .normal, normal, RightVector, -alpha
  .dist = -DirectX.VectorDotProduct(.normal, Camera)
 End With
 
 'left plane
With OctreePlanes(3)
  normal = Vector(RightVector.x, RightVector.y, RightVector.z)
  DirectX.VectorRotate .normal, normal, UpVector, -beta
  .dist = -DirectX.VectorDotProduct(.normal, Camera)
 End With
 
 'right plane
With OctreePlanes(4)
  normal = Vector(-RightVector.x, -RightVector.y, -RightVector.z)
  DirectX.VectorRotate .normal, normal, UpVector, beta
  .dist = -DirectX.VectorDotProduct(.normal, Camera)
End With
 
 'near plane
With OctreePlanes(5)
 .normal = LookVector
 .dist = -DirectX.VectorDotProduct(.normal, Camera)
End With
Dim TempVector As D3DVECTOR
 
 'Z far plane
With OctreePlanes(6)
 'DirectX.VectorScale TempVector, LookVector, FarPlane
 .normal = LookVector
 .dist = OctreePlanes(6).dist * FarPlane
End With

For i = 1 To 6
 BackBuffer.DrawText 50, 100 + i * 15, CStr(OctreePlanes(i).normal.x) + " " + CStr(OctreePlanes(i).normal.y) + " " + CStr(OctreePlanes(i).normal.z) + " " + CStr(OctreePlanes(i).dist), False
Next i
End Sub

Private Function GetFaceCenter(vert() As D3DVERTEX, numvert As Integer) As D3DVECTOR
 Dim temp As D3DVECTOR
 temp = Vector(0, 0, 0)
 For i = 1 To numvert
  temp = VAdd(temp, Vector(vert(i).x, vert(i).y, vert(i).z))
 Next i
 temp = VScale(temp, 1 / numvert)
End Function


' BuildOctree is a recursive sub
'
'  - if it's the rootnode, calculate the general bounding box (or bounding cube)
'  - then check each face to see if it's in the box
'      * if yes, add the face to the list
'      * otherwise, the face is already in a other node, so bye bye Face !
'  - if there are more than MaxNodePerFace faces in the node, divide the node in 8 others
' nodes (and cubes...) and restart the sub with the new nodes.
'  - if there are less than MaxNodePerFace faces in the node, we have a leaf and
' this part of the tree is finish.
'
' I know that it's very difficult to understand

Public Function BuildOctree(Node As Integer, FaceIndex() As Integer, numface, Optional root As Boolean) As Boolean
  
  
  Dim FaceCenter As D3DVECTOR, NodeFace() As Integer
  ReDim NodeFace(numface)
  
  If IsMissing(root) = True Then
   root = True
  Else
   root = False
  End If
  
  'if it's the first time we call the sub : calculate the whole bounding box
  If root = True Then
    Dim min As D3DVECTOR
    Dim max As D3DVECTOR
    ReDim Octree.Node(1)
    
    min.x = 9999999
    min.y = 9999999
    min.z = 9999999
    max.x = -9999999
    max.y = -9999999
    max.z = -9999999
    
    For i = 1 To Octree.FaceCount
     'calculate bounding box from faces' center
     FaceCenter = GetFaceCenter(Octree.Face(i).Vertex, UBound(Octree.Face(i).Vertex))
     With Octree.Face(i)
       If FaceCenter.x < min.x Then min.x = FaceCenter.x
       If FaceCenter.y < min.y Then min.y = FaceCenter.y
       If FaceCenter.z < min.z Then min.z = FaceCenter.z
       
       If FaceCenter.x > max.x Then max.x = FaceCenter.x
       If FaceCenter.y > max.y Then max.y = FaceCenter.y
       If FaceCenter.z > max.z Then max.z = FaceCenter.z
     End With
    Next i
    'we search a cube ! same dimensions !
    If min.x < min.y And min.x < min.z Then min.y = min.x: min.z = min.x
    If min.y < min.x And min.y < min.z Then min.x = min.y: min.z = min.y
    If min.z < min.x And min.z < min.z Then min.x = min.z: min.y = min.z
   
    If max.x > max.y And max.x > max.z Then max.y = max.x: max.z = max.x
    If max.y > max.x And max.y > max.z Then max.x = max.y: max.z = max.y
    If max.z > max.x And max.z > max.z Then max.x = max.z: max.y = max.z
    
    SetBox Node, min, max
  End If
  min = Octree.Node(Node).bbox.min
  max = Octree.Node(Node).bbox.max
  'check if each face is in our octree node :
  For i = 1 To numface
   With Octree.Face(FaceIndex(i))
      FaceInNode = False
      'test if the face center or a face vertex is in the bounding box :
      FaceCenter = GetFaceCenter(.Vertex, UBound(.Vertex))
      If FaceCenter.x >= min.x And FaceCenter.x <= max.x And FaceCenter.y >= min.y And FaceCenter.y <= max.y And FaceCenter.z >= min.z And FaceCenter.z <= max.z Then
       'we have a hit !!
       FaceInNode = True
       GoTo AddFace
      End If
      For v = 1 To UBound(.Vertex)
        If .Vertex(v).x >= min.x And .Vertex(v).x <= max.x And .Vertex(v).y >= min.y And .Vertex(v).y <= max.y And .Vertex(v).z >= min.z And .Vertex(v).z <= max.z Then
         'we have a hit
         FaceInNode = True
         GoTo AddFace
        End If
      Next v
AddFace:
     If FaceInNode = True Then
      num_face = num_face + 1
      NodeFace(num_face) = FaceIndex(i)
      ReDim Preserve Octree.Node(Node).faces(num_face)
      Octree.Node(Node).faces(num_face) = FaceIndex(i)
     End If
   End With
  Next i
  
  If num_face = 0 Then
    'no face, so let's stop recursivity for this node
    Exit Function
  End If
  
  
  If num_face > Octree.MaxFacePerNode Then
   'there are too many faces in the node :
   'divide the node in 8 other nodes
   numbernode = UBound(Octree.Node)
   ReDim Preserve Octree.Node(numbernode + 8)
   Dim Length As D3DVECTOR
   Length = VSubtract(max, min)
   For i = 1 To 8
     Octree.Node(Node).OctNode(i) = numbernode + i
     Octree.Node(numbernode + i).Parent = Node
     'create the new bounding boxes :
     With Octree.Node(numbernode + i).bbox
        
        Select Case i
         Case 1
          .min = Vector(min.x, min.y, min.z)
          .max = Vector(min.x + Length.x / 2, min.y + Length.y / 2, min.z + Length.z / 2)
         Case 2
          .min = Vector(min.x, min.y, min.z + Length.z / 2)
          .max = Vector(min.x + Length.x / 2, min.y + Length.y / 2, max.z)
         Case 3
          .min = Vector(min.x, min.y + Length.y / 2, min.z + Length.z / 2)
          .max = Vector(min.x + Length.x / 2, max.y, max.z)
         Case 4
          .min = Vector(min.x, min.y + Length.y / 2, min.z)
          .max = Vector(min.x + Length.x / 2, max.y, min.z + Length.z / 2)
         Case 5
          .min = Vector(min.x + Length.x / 2, min.y, min.z)
          .max = Vector(max.x, min.y + Length.y / 2, min.z + Length.z / 2)
         Case 6
          .min = Vector(min.x + Length.x / 2, min.y, min.z + Length.z / 2)
          .max = Vector(max.x, min.y + Length.y / 2, max.z)
         Case 7
          .min = Vector(min.x + Length.x / 2, min.y + Length.y / 2, min.z)
          .max = Vector(max.x, max.y, min.z + Length.z / 2)
         Case 8
          .min = Vector(min.x + Length.x / 2, min.y + Length.y / 2, min.z + Length.z / 2)
          .max = Vector(max.x, max.y, max.z)
        End Select
        SetBox numbernode + i, .min, .max
     End With
   Next i
   For i = 1 To 8
     'Restart the recursive stuff with the new nodes !
     BuildOctree numbernode + i, NodeFace(), num_face, False
   Next i
  Else
   'we have a leaf !!
   'yoopi !
   'nothing to do !
  End If
  
  
End Function
Function IsBoxOutside(Box As OctreeBox, Plane As OctreePlane) As Boolean
  
  IsBoxOutside = True
  
  For i = 1 To 8
    If IsInsidePlane(Box.corner(i), Plane) = True Then IsBoxOutside = False
  Next i

End Function

Function IsBoxOutsideFrustum(Box As OctreeBox) As Boolean
   IsBoxOutsideFrustum = False
   For i = 3 To 5
   If IsBoxOutside(Box, OctreePlanes(i)) = False Then IsBoxOutsideFrustum = True
   Next i

End Function
Function IsInsidePlane(Vector As D3DVECTOR, Plane As OctreePlane) As Boolean
 If DirectX.VectorDotProduct(Vector, Plane.normal) - Plane.dist >= 0 Then
   IsInsidePlane = True
 Else
   IsInsidePlane = False
 End If
End Function

Function IsSphereOutside(Box As OctreeBox) As Boolean
' AddLog CStr(Box.min.x) + " " + CStr(Box.min.y) + " " + CStr(Box.min.z) + " / " + CStr(Box.max.x) + " " + CStr(Box.max.y) + " " + CStr(Box.max.z)
' AddLog CStr(device.ComputeSphereVisibility(Vector((Box.min.x + Box.max.x) / 2, ((Box.min.y + Box.max.y) / 2), ((Box.min.z + Box.max.z) / 2)), (Box.max.z - Box.min.z) / 2))
 clipstatus = device.ComputeSphereVisibility(Vector((Box.min.x + Box.max.x) / 2, ((Box.min.y + Box.max.y) / 2), ((Box.min.z + Box.max.z) / 2)), (Box.max.z - Box.min.z) / 2)
 If clipstatus And &HFFF000 Then
  IsSphereOutside = True
 Else
  IsSphereOutside = False
 End If
End Function


Sub OctreeLoadTextures()
 On Error GoTo apprend

'Check if we have to load textures with "number.bmp" or with "name.bmp"
If OctreeTextureMode = 2 Then
  UseNumberFilename = True
Else
  UseNumberFilename = False
End If
 
 For i = 0 To UBound(Octree.texture) - 1
   
   If IsMissing(TextureWidth) Then
     TextureWidth = Octree.texture(i).width
   End If
   If IsMissing(TextureHeight) Then
     TextureHeight = Octree.texture(i).height
   End If
   
   FreeTexture = SearchFreeTexture
   
   If UseNumberFilename = True Then
    Set texture(FreeTexture) = LoadSimpleTexture(CStr(i) + ".bmp", OctreeTextureWidth, OctreeTextureHeight)
   Else
    Set texture(FreeTexture) = LoadSimpleTexture(Octree.texture(i).texture + ".bmp", OctreeTextureWidth, OctreeTextureHeight)
   End If
   TextureName(FreeTexture) = UCase(Octree.texture(i).texture + ".bmp")
   TextureIndex(i) = FreeTexture
   
   'check if this texture is a texture for the triggers (Half-Life only i think)
   If InStr(Octree.texture(i).texture, "AAATRIGGER") Then
     TriggerTexture = i
   End If
   
   'Half-Life : if a texture name begins with "{", then the texture is transparent
   If Left(BSP.MipTexture(i).TextureName, 1) = "{" Then
      ERrStr = "Set Color Key"
      Dim ppf As DDPIXELFORMAT
      BackBuffer.GetPixelFormat ppf
      Dim ck As DDCOLORKEY
      ck.high = ppf.lBBitMask
      ck.low = ppf.lBBitMask
      device.SetRenderState D3DRENDERSTATE_COLORKEYBLENDENABLE, True
      device.SetRenderState D3DRENDERSTATE_COLORKEYENABLE, True
   End If
  Next i
  TextureLoaded = True
Exit Sub

apprend:
 AddLog CStr(Date) + " " + CStr(Time) + " Error in LoadTextures " + Err.Description + " " + CStr(Err.Number) + " " + CStr(i) + " " + ERrStr
 Resume Next
End Sub

Sub ModuleOctreeRender()
On Error GoTo trat
 Dim FTDraw As Integer, t As String
 
 GetClippingPlanes
 
 'reset face-to-draw indexes
 ReDim Preserve OctreeFaces(1)
 NumFaceToDraw = 0
 
 ERrStr = "WalkTree"
 'Walk octree
 WalkTree 1
 
 'Render the visible faces:
 AncTexture = -1
 ERrStr = "Draw face"
 For i = 1 To NumFaceToDraw
   FTDraw = OctreeFaces(i)
   If AncTexture <> TextureIndex(Octree.Face(FTDraw).texture) Then
    device.SetTexture 0, texture(TextureIndex(Octree.Face(FTDraw).texture))
   End If
   device.DrawPrimitive D3DPT_TRIANGLEFAN, D3DFVF_VERTEX, Octree.Face(FTDraw).Vertex(0), UBound(Octree.Face(FTDraw).Vertex), D3DDP_DEFAULT
   t = t + CStr(frdraw) + " "
 Next i
 
 ERrStr = "Draw info"
 BackBuffer.DrawText 5, 400, CStr(NumFaceToDraw) + " faces", False
 BackBuffer.DrawText 5, 420, t, False
 
 '------------
 ''The End !!!
 '------------
 Exit Sub
trat:
 'AddLog "erreur in oct " + Err.Description + " " + CStr(Err.Number) + " " + ERrStr
 Resume Next
End Sub

Sub PushNodeFaces(Node As Integer)
 On Local Error Resume Next
 
  For i = 1 To UBound(Octree.Node(Node).faces)
   For k = 1 To NumFaceToDraw
     If OctreeFaces(k) = Octree.Node(Node).faces(i) Then GoTo sui
   Next k
   NumFaceToDraw = NumFaceToDraw + 1
   ReDim Preserve OctreeFaces(NumFaceToDraw)
   OctreeFaces(NumFaceToDraw) = Octree.Node(Node).faces(i)
sui:
  Next i
  Err.Clear
End Sub


Public Function SearchFreeTexture() As Integer
  For i = 1 To 100
   If TextureName(i) = "" Then
    ok = i
   End If
  Next i
  SearchFreeTexture = ok
  
End Function

Private Sub SetBox(Node, min As D3DVECTOR, max As D3DVECTOR)
   Octree.Node(Node).bbox.min = min
   Octree.Node(Node).bbox.max = max
   
   With Octree.Node(Node).bbox
    .corner(1).x = min.x
    .corner(2).x = min.x
    .corner(3).x = min.x
    .corner(4).x = min.x
    .corner(5).x = max.x
    .corner(6).x = max.x
    .corner(7).x = max.x
    .corner(8).x = max.x
    
    .corner(1).y = min.y
    .corner(2).y = min.y
    .corner(3).y = max.y
    .corner(4).y = max.y
    .corner(5).y = min.y
    .corner(6).y = min.y
    .corner(7).y = max.y
    .corner(8).y = max.y
    
    .corner(1).z = min.z
    .corner(2).z = max.z
    .corner(3).z = min.z
    .corner(4).z = max.z
    .corner(5).z = min.z
    .corner(6).z = max.z
    .corner(7).z = min.z
    .corner(8).z = max.z
  End With
   
End Sub
Sub WalkTree(Node As Integer)
 On Error GoTo re
 'AddLog "Walktree " + CStr(Node)
 ERrStr = "Check node"
 If IsSphereOutside(Octree.Node(Node).bbox) = False Then
   'this parent node is not in frustum view, so are its children
   Exit Sub
 Else
  ERrStr = "Check leaf"
   'if this node is a leaf, then draw the faces !
   If IsLeaf(Node) = True Then
     ERrStr = "Push"
     PushNodeFaces Node
     Exit Sub
   End If
   
   'if all the children nodes are in the frustum, then draw the faces !
   ERrStr = "Check children"
   With Octree.Node(Node)
   For i = 1 To 8
    If IsSphereOutside(Octree.Node(.OctNode(i)).bbox) = True Then GoTo NoFrustumNode
   Next i
   End With
   
   'all the children are in view
   ERrStr = "push"
   PushNodeFaces Node
   Exit Sub
   
NoFrustumNode:
   ERrStr = "Recursive"
   'there is some visible children nodes
   For i = 1 To 8
    WalkTree Octree.Node(Node).OctNode(i)
   Next i
 End If
 Exit Sub
re:
 'AddLog "Walk tree error :" + CStr(Err.Number) + " " + ERrStr + " " + CStr(Node)
End Sub


Function IsLeaf(Node As Integer) As Boolean
  For i = 1 To 8
  If Octree.Node(Node).OctNode(i) > 0 Then IsLeaf = False: Exit Function
  Next i
  IsLeaf = True
End Function
